/*jslint browser:true*/
/*global define*/

define([
    'testsuite/qunit',
    'jquery',
    'ep',
    'de_epages/presentation/tray/setup',

    '$ready!'
], function (QUnit, $, ep, traySetup) {

    'use strict';

    var node,
        fixture = $('#qunit-fixture'),
        data = {
            'tableHead': [{
                'isSortable': 0,
                'name': '',
                'id': 'ObjectID',
                'isSelectable': 1
            }, {
                'isSortable': 1,
                'name': 'Product number',
                'id': 'Alias',
                'isSelectable': 0
            }, {
                'isSortable': 1,
                'name': '',
                'id': 'NameOrAlias',
                'isLinkable': 1,
                'isSelectable': 0
            }, {
                'isSortable': 0,
                'name': '',
                'id': 'VariationAttr',
                'isSelectable': 0
            }, {
                'isSortable': 1,
                'name': 'List price',
                'id': 'ListPrice',
                'isSelectable': 0
            }],
            'totalNumber': '22',
            'orderBy': 'NameOrAlias',
            'lastPage': 2,
            'orderDesc': 0,
            'tableContent': {
                '1': [{
                    'Alias': 'be_40401',
                    'Parents': [],
                    'ObjectID': 17144,
                    'VariationAttr': null,
                    'NameOrAlias': 'Berghaus Paclite Jacket - Men',
                    'hasSubTable': 1,
                    'ListPrice': '199.95 €'
                }, {
                    'Alias': 'be_40402',
                    'Parents': [],
                    'ObjectID': 17156,
                    'VariationAttr': null,
                    'NameOrAlias': 'Berghaus Paclite Jacket - Women',
                    'hasSubTable': 1,
                    'ListPrice': '199.95 €'
                }, {
                    'Alias': 'vs_3202112018',
                    'Parents': [],
                    'ObjectID': 17215,
                    'VariationAttr': null,
                    'NameOrAlias': 'Black Bear Gemini',
                    'ListPrice': '29.95 €'
                }, {
                    'Alias': 'cg_0101004270',
                    'Parents': [],
                    'ObjectID': 17137,
                    'VariationAttr': null,
                    'NameOrAlias': 'Camping AZ CV270 valve gas canister',
                    'ListPrice': '3.95 €'
                }, {
                    'Alias': 'cg_0101104470',
                    'Parents': [],
                    'ObjectID': 17138,
                    'VariationAttr': null,
                    'NameOrAlias': 'Camping AZ CV470 valve Gas canister',
                    'ListPrice': '7.95 €'
                }, {
                    'Alias': 'de_3201212002',
                    'Parents': [],
                    'ObjectID': 17212,
                    'VariationAttr': null,
                    'NameOrAlias': 'Deuter Hydro 2.0',
                    'ListPrice': '74.95 €'
                }, {
                    'Alias': 'de_3203104010',
                    'Parents': [],
                    'ObjectID': 17213,
                    'VariationAttr': null,
                    'NameOrAlias': 'Deuter Kangaroo',
                    'ListPrice': '99.95 €'
                }, {
                    'Alias': 'de_3206199010',
                    'Parents': [],
                    'ObjectID': 17216,
                    'VariationAttr': null,
                    'NameOrAlias': 'Deuter Teddy Bear',
                    'ListPrice': '26.95 €'
                }, {
                    'Alias': 'er_7142303001',
                    'Parents': [],
                    'ObjectID': 17139,
                    'VariationAttr': null,
                    'NameOrAlias': 'Edelrid Black Bear Rope',
                    'ListPrice': '1.45 €'
                }],
                '2': [{
                    'Alias': 'be_40401',
                    'Parents': [],
                    'ObjectID': 17144,
                    'VariationAttr': null,
                    'NameOrAlias': 'Berghaus Paclite Jacket - Men',
                    'hasSubTable': 1,
                    'ListPrice': '199.95 €'
                }, {
                    'Alias': 'be_40402',
                    'Parents': [],
                    'ObjectID': 17156,
                    'VariationAttr': null,
                    'NameOrAlias': 'Berghaus Paclite Jacket - Women',
                    'hasSubTable': 1,
                    'ListPrice': '199.95 €'
                }]
            }
        };

    ep.ajax = function (options) {
        return {
            done: function (f) {
                if (ep.ajax.callback) {
                    ep.ajax.callback(f, options);

                    return {
                        fail: function(data) {
                            $.noop();
                        }
                    };
                } else {
                    throw new Error('No callback for ajax mockup defined');
                }
            }
        };
    };

    QUnit.module('de_epages/presentation/tray/setup', {
        setup: function () {
            fixture.html('<form><button type="button" class="ep-tray">Tray</button></form>');
            node = fixture.find('button.ep-tray');
            fixture.find('form').on('submit', function (event) {
                event.preventDefault();
                return false;
            });
        },

        teardown: function () {
                fixture.empty();
        }
    });

    QUnit.asyncTest('Selecting nothing should do nothing', 5, function () {
        node.addClass('Disabled');
        // Getting some objects
        ep.ajax.callback = function (f, options) {
            f(data);

            QUnit.equal(
                options.data.ViewAction,
                'JSONSearchProductsToInsert',
                'The JSONSearchProductsToInsert view action should be used to fetch tray objects'
            );
        };

        traySetup({
            node: node,
            dialog: {
                title: 'This is a fancy dialog'
            },
            pageId: 1337,
            changeAction: 'JSONInsertIt',
            smartSearch: {
                source: ['Foo', 'Bar'],
                placeholder: 'My placeholder',
                name: 'NewObject'
            },
            tabs: {
                products: {
                    id: 'products',
                    action: 'JSONSearchProductsToInsert',
                    objectName: 'ProductIDs'
                }
            },
            preSmartSearchSubmitHook: function () {
                QUnit.ok(
                    true,
                    'The preSmartSearchSubmitHook was fired'
                );
            }
        });

        // We're loading the finder async from the rest of the modules, so we have to wait until the setup has that thing ready
        setTimeout(function () {
            var smartsearch = node.siblings('input'),
                e = $.Event("keypress");

            QUnit.ok(
                smartsearch.prop('placeholder') && smartsearch.prop('name') && smartsearch.hasClass('Disabled'),
                'Smartsearch is initialized and has the correct attributes and classes'
            );

            e.keyCode = 13; // Return
            // Try to trigger the preSmartSearchSubmitHook
            smartsearch.trigger(e);

            node.trigger('click');

            setTimeout(function () {
                QUnit.equal(
                    $('.ui-dialog').css('display'),
                    'block',
                    'A dialog should be opened'
                );

                // Overwrite ajax callback - Do nothing
                ep.ajax.callback = function (f, options) {
                    QUnit.ok(false, 'The callback should not make an AJAX request when no item is selected');
                };

                $('div.ui-dialog-buttonpane button').trigger('click');
                QUnit.equal($('div.ui-dialog').css('display'), 'none', 'No dialog should be opened');

                QUnit.start();
            }, 2444);
        }, 2444);
    });

    QUnit.asyncTest('Selecting something should issue the correct AJAX request', 4, function () {
        traySetup({
            node: node,
            dialog: {
                title: 'This is a fancy dialog'
            },
            pageId: 1337,
            changeAction: 'JSONInsertIt',
            tabs: {
                products: {
                    id: 'products',
                    action: 'JSONSearchProductsToInsert',
                    objectName: 'ProductIDs',
                    additionalContent: '<input type="checkbox" name="myCheckbox" id="the-additional-checkbox" />',
                    parseAdditionalContent: function (c) {
                        QUnit.ok(true, 'We should have a chance to parse the additional content');
                        return {
                            'MyCheckbox': c.myCheckbox ? true : false
                        };
                    }
                }
            },
            postHook: function () {
                QUnit.ok(true, 'The post hook should be called');
                return {
                    'AnotherCustomParameter': 'FooBar'
                };
            }
        });

        // Overwrite ajax callback - Get data
        ep.ajax.callback = function (f, options) {
            f(data);
        };

        setTimeout(function () {
            node.trigger('click');

            $('input#17137').trigger('click');
            $('input#the-additional-checkbox').trigger('click');

            // Overwrite ajax callback - Send data
            ep.ajax.callback = function (f, options) {
                QUnit.deepEqual({
                    'type': 'POST',
                    'dataType': 'json',
                    'data': [{
                        'name': 'ChangeAction',
                        'value': 'JSONInsertIt'
                    }, {
                        'name': 'ViewAction',
                        'value': 'JSONViewResponse'
                    }, {
                        'name': 'ChangeObjectID',
                        'value': 1337
                    }, {
                        'name': 'ProductIDs',
                        'value': 17137
                    }, {
                        'name': 'MyCheckbox',
                        'value': true
                    }, {
                        'name': 'AnotherCustomParameter',
                        'value': 'FooBar'
                    }]
                }, options, 'The tray should issue the correct request');

                f();
            };

            fixture.find('form').on('submit', function (e) {
                e.preventDefault();
                QUnit.ok(true, 'The form should be submitted');
            });

            $('div.ui-dialog-buttonpane button').trigger('click');

            QUnit.start();
        }, 444);
    });

});