/*
 * @copyright   © Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module      de_epages.presentation.inc.batchactions
 */

/*jslint browser: true, nomen: true, regexp: true, evil: true, forin: true*/
/*global define, epages*/

define("de_epages/presentation/inc/batchactions", [
    "jquery",
    "ep",
    "underscore",
    "util/storage",

    "ep/fn/holdontop"
], function ($, ep, _, storage) {

    'use strict';

    var batchactions = {
            stack: []
        },

        fnBatchActionStack = batchactions.stack,

        $firstContentList = $('.ContentList')
            .filter(function () {
                return !!$(this).prevAll('.BatchActionsBar').length;
            })
            .first(),

        selectedCheckboxes = storage.sessionStorage('ep_selectedCheckboxes'),

        selectedValues,

        selectedTableRows,

        $selectedTableRows,

        toggleDefaultChangeAction = function (form, isDisabled) {
            // Just so some jQuery form widget will not stop us from submitting - start.
            // Prevent *form* from validating the *input*s once again.
            $(form)
                .attr('formnovalidate', true);
            // Just so some jQuery form widget will not stop us from submitting - end.
            $(form.elements)
                .filter('[name=ChangeAction][type=hidden]')
                .prop('disabled', isDisabled);
        },

        // Activate/Deactivate BatchAction buttons
        toggleBatchActionAvailability = function () {
            var contentList = this;

            // ContentList widget natively delays checkbox setting
            setTimeout(function () {
                var dynamicButtons = $('.EnableTableSelection'),
                    activeRows = $(contentList)
                        .find('tbody tr td:first-child input[type=checkbox]:checked')
                        .closest('tr'),
                    rowCount,
                    rowsMarkedNew,
                    key;

                if (activeRows.length) {
                    dynamicButtons.removeClass('BatchActionDisabled');
                    dynamicButtons.find('input')
                        .removeAttr('disabled');
                } else {
                    dynamicButtons.addClass('BatchActionDisabled');
                    dynamicButtons.find('input')
                        .attr('disabled', 'true');
                }

                //set Visibility and Mark active
                if (activeRows.length) {
                    if (!(activeRows.find('i[class*=_inactive]:first,img[class*=imgOff]:first, img[src*=_inactive]:first')
                        .length + activeRows.filter('.Opacity50')
                        .length)) {
                        $('.ep-batchActions-groupImage-View')
                            .parent()
                            .addClass('ep-batchActions-groupImage-active');
                    } else {
                        $('.ep-batchActions-groupImage-View')
                            .parent()
                            .removeClass('ep-batchActions-groupImage-active');
                    }

                    rowCount = activeRows.length;
                    rowsMarkedNew = activeRows.find('i[class*=ico_s_isnew]:first').length;

                    if (rowCount === rowsMarkedNew) {
                        $('.ep-batchActions-groupImage-Mark')
                            .parent()
                            .addClass('ep-batchActions-groupImage-active');
                    } else {
                        $('.ep-batchActions-groupImage-Mark')
                            .parent()
                            .removeClass('ep-batchActions-groupImage-active');
                    }
                }

                if (fnBatchActionStack.length) {
                    for (key in fnBatchActionStack) {
                        fnBatchActionStack[key]();
                    }
                }
            }, 1);

        },

        line,

        inputs,
        addSave,
        submit,
        setAdd,
        setSave,
        onFocus;

    /* If sessionStorage contains info about selected Checkboxes,
       then select the corresponding table rows and do a highlighting effect */
    selectedValues = selectedCheckboxes ? _.pluck(selectedCheckboxes, 'value') : [];
    selectedTableRows = [];

    $firstContentList
        .find('input:checkbox')
        .each(function () {
            var $elm = $(this),
                $row;

            if (_.indexOf(selectedValues, $elm.val()) !== -1 || $elm.data('highlight')) {
                $row = $elm.closest('tr');

                selectedTableRows.push($row.get(0));
            }
        });

    // Get jQuery object for elements stack
    $selectedTableRows = $(selectedTableRows)
        .addClass('HideSelectedRefresh');

    // set Timeout-functions, necessary for animation (see "table.ContentList tbody tr.ShowSelectedRefresh td span" @ BackofficeStyle.css)
    setTimeout(function () {
        $selectedTableRows
            .addClass('ShowSelectedRefresh');
    }, 1);

    // necessary to remove css class when animation has ended
    setTimeout(function () {
        $selectedTableRows
            .removeClass('ShowSelectedRefresh')
            .removeClass('HideSelectedRefresh');
    }, 2500);

    /* Clear sessionStorage again */
    storage.sessionClear('ep_selectedCheckboxes');

    $('.BatchActions > li')
        .each(function () {
            /* initialze BatchAction menu */

            var elem = $(this),
                label = elem.children('label'),
                items = elem.find('li'),

                textElem,
                text,

                titleNode,
                heightBefore,
                heightAfter;

            if (items.length <= 1) {
                textElem = items.find('> label > div:first');
                text = $.trim(textElem.text());

                if (text.length > 25) {
                    textElem.attr('title', text)
                        .text(ep.scrunch(text, 25));
                }

                label.children('input')
                    .before(items.find('input'))
                    .remove();
                label.find('> div:last')
                    .before(textElem)
                    .remove();
                items.parent()
                    .remove();
            } else {
                label.children('input')
                    .before(items.find('input')
                        .clone(true));
                items.parent()
                    .hover(
                        function () {
                            label.children('input')
                                .addClass('active');
                        },
                        function () {
                            label.children('input')
                                .removeClass('active');
                        }
                    );

                label.find('> div:first')
                    .addClass('ep-batchActions-hasMenu');

                label.find('> div:last')
                    .append('<span class="ep-batchActions-arrow">&#9660;</span>')
                    .addClass('ep-batchActions-hasMenu')
                    .on('click', function (evt) {
                        evt.preventDefault();
                        evt.stopPropagation();
                        var $elm = $(this);

                        if (!$elm.parents('label').hasClass('BatchActionDisabled')) {
                            $elm.parents('li').toggleClass('showMenu');
                        }
                    });

                //check if title is longer than one line
                titleNode = label.find('.batch-action-title');

                heightBefore = titleNode.height();

                titleNode.css({
                    whiteSpace: 'nowrap'
                });

                heightAfter = titleNode.height();

                titleNode.css({
                    whiteSpace: 'normal'
                });

                if (heightBefore !== heightAfter) {
                    titleNode.addClass('batch-action-long-title');
                }

                $('body')
                    .on('click', function () {
                        $('.showMenu').removeClass('showMenu');
                    });

                elem.hTimer = undefined;
                elem.hover(
                    function () {
                        // clear timeout if mouse enters menue again
                        if (elem.hTimer !== undefined) {
                            window.clearTimeout(elem.hTimer);
                            elem.hTimer = undefined;
                        }
                    },
                    function () {
                        var $this = $(this);

                        // close menue with delay
                        elem.hTimer = window.setTimeout(function () {
                            //remove 'showMenu' if unhover event for whole li is called
                            $this.removeClass('showMenu');
                        }, 500);
                    }
                );
            }
        })
        .find('input:radio').on('change', function () {
            /* bind BatchAction events */
            var $elm = $(this),
                scriptAction = $elm.data('scriptAction'),
                inputValue,
                selectedInputs,
                filteredSelections;

            if ($elm.parent().hasClass('BatchActionDisabled')) {
                return;
            }

            inputValue = $elm.val();

            // 'SetInvisibleListedObjects' used only for Categories-section
            // If the currently marked row is set visible, then set it invisible.
            // detect if the row is visible via class which ist set to the icon or image
            if (inputValue.match(/Set.*(?!Not)Visible/g) || inputValue === 'SetInvisibleListedObjects') {
                if (!$firstContentList.find('tr.RowSelected i[class*=fa-eye-slash]').length && $elm.next().is('input')) {
                    $elm.val($elm.next().val());
                }
            }

            if (inputValue === 'SetProductsNew') {
                // no other option to get the *new* flag, sorry
                if ($firstContentList.find('tr.RowSelected i[class*=certificate]').length === $firstContentList.find('tr.RowSelected').length) {
                    $elm.val($elm.next().val());
                }
            }

            toggleDefaultChangeAction(this.form, true);

            if (scriptAction) {
                //try to avoid error
                try {
                    //used to exectue batch action saved in variable
                    eval('(' + scriptAction + ').call(this,arguments[0]);');
                } catch (error) {
                    toggleDefaultChangeAction(this.form, false);
                    $.error(error);
                }
            } else {
                /* Filtering the selected items (Variations products can't be set to new) */
                selectedInputs = $firstContentList.find('input[name=ListObjectID]:checked');
                filteredSelections = [];

                if (inputValue === 'SetProductsNew' || inputValue === 'SetProductsNotNew') {
                    selectedInputs.each(function () {
                        if (!$(this).closest('tr').find('a[href*="ViewAction=MBO-ViewVariations"]').length) {
                            filteredSelections.push(this);
                        }
                    });
                } else {
                    filteredSelections = selectedInputs;
                }

                storage.sessionStorage('ep_selectedCheckboxes', $(filteredSelections).serializeArray());

                // Just so some jQuery form widget will not stop us from submitting - start.
                // Remove pseudo *:invalid* property from *input* elements.
                $(this.form)
                    .formInput()
                    .filter(':invalid')
                    .each(function (i, element) {
                        /*jslint unparam: true*/
                        element.formInvalid = false;
                    });

                // set defaultAttributes of changed elements to their current values => no confirmations dialog appears when leaving the page
                $(this.form)
                    .formInput()
                    .filter(':changed')
                    .each(function (i, element) {
                        /*jslint unparam: true*/
                        var $input = $(element),
                            checkboxOrRadioButton = false;

                        if (element.getAttribute("type") === 'checkbox' || element.getAttribute("type") === 'radio') {
                            checkboxOrRadioButton = true;
                        }

                        $input.setDefault(checkboxOrRadioButton ? $input.is(':checked') : $input.val());
                    });

                // Just so some jQuery form widget will not stop us from submitting - end.
                $(this.form).trigger('submit');
            }
        });

    /* show BatchActionBar always on top */
    $('.BatchActionsBar')
        .holdOnTop('top');

    // select rows if checkbox is checked on start up
    $('.ContentList tbody tr td:first-child input[type=checkbox]:checked')
        .closest('tr')
        .addClass('RowSelected');

    $firstContentList.on('mouseup', toggleBatchActionAvailability);
    $firstContentList.on('keyup', toggleBatchActionAvailability);
    toggleBatchActionAvailability.call($firstContentList);

    /* initialize NewEntry table row */
    line = $('tr.NewEntry');

    // if a row for a new entry exists
    if (line.length) {
        inputs = line.find('input');

        addSave = line.find('.Checkbox > .PlusIcon');

        submit = inputs.closest("form").find('input[type=submit]');

        setAdd = function () {
            // search for dynamically added inputs
            line.find('input:visible')
                .first()
                .trigger('focus');
        };

        setSave = function () {
            inputs = $('tr.NewEntry').find('input:visible');

            if (inputs.length) {
                //check and clear epages.widget.ButtonChangeForms to avoid leave page warning on submit
                if (epages.widget.ButtonChangeForms && epages.widget.ButtonChangeForms.length) {
                    epages.widget.ButtonChangeForms = [];
                }

                $(inputs[0].form)
                    .trigger({
                        type: 'submit',
                        dojoconnect: true
                    });
            }
        };

        onFocus = function () {
            if (!submit.length) {
                submit = $('<input type="submit" class="HideElementSoft" />').insertAfter(addSave);
            }

            inputs.off('focus', onFocus);

            addSave.off()
                .on('click', setSave)
                .toggleClass('ico_s_add', false)
                .toggleClass('ico_s_save', true);
        };

        addSave.on('click', setAdd);
        line.on('focus', 'input', onFocus);
    }

    return batchactions;
});
